import { Extension } from 'resource:///org/gnome/shell/extensions/extension.js';
import { panel } from 'resource:///org/gnome/shell/ui/main.js';
import GLib from 'gi://GLib';

export default class IndicatorVisibilityToolExtension extends Extension {
    enable() {
        this._settings = this.getSettings('org.gnome.shell.extensions.IndicatorVisibilityTool');
        this._settingsChangedId = this._settings.connect('changed', this._updateIndicators.bind(this));

        this._count = 0;
        this._signals = new Map();
 
        this._interval = GLib.timeout_add_seconds(GLib.PRIORITY_DEFAULT, 1, () => {
            if (!this._indicators && this._count++ < 16) {
                this._initIndicators();
                return GLib.SOURCE_CONTINUE;
            }

            if (this._indicators) this._updateIndicators();
            return GLib.SOURCE_REMOVE;
        });
    }

    disable() {
        if (this._settingsChangedId) this._settings.disconnect(this._settingsChangedId);
        if (this._interval) GLib.source_remove(this._interval);

        this._signals.forEach((signalId, indicator) => {
            if (indicator && signalId) indicator.disconnect(signalId);
        });
        this._signals.clear();

        this._count = this._settings = this._indicators = this._settingsChangedId = this._interval = null;
    }

    _initIndicators() {
        if (!panel.statusArea || !panel.statusArea.quickSettings) return;

        this._indicators = {
            'system-mode':        panel.statusArea.quickSettings._system,
            'unsafe-mode':        panel.statusArea.quickSettings._unsafeMode,
            'volume-output-mode': panel.statusArea.quickSettings._volumeOutput,
            'rfkill-mode':        panel.statusArea.quickSettings._rfkill,
            'bluetooth-mode':     panel.statusArea.quickSettings._bluetooth,
            'network-mode':       panel.statusArea.quickSettings._network,
            'night-light-mode':   panel.statusArea.quickSettings._nightLight,
            'thunderbolt-mode':   panel.statusArea.quickSettings._thunderbolt,
            'location-mode':      panel.statusArea.quickSettings._location,
            'volume-input-mode':  panel.statusArea.quickSettings._volumeInput,
            'camera-mode':        panel.statusArea.quickSettings._camera,
            'remote-access-mode': panel.statusArea.quickSettings._remoteAccess
        };

        if (Object.values(this._indicators).some(indicator => !indicator)) {
            this._indicators = null;
            return;
        }

        Object.entries(this._indicators).forEach(([key, indicator]) => {
            if (!indicator) return;
            const target = indicator._indicator || indicator;

            if (!this._signals.has(target)) {
                const signalId = target.connect('notify::visible', () => this._updateIndicatorVisibility(key, target));
                this._signals.set(target, signalId);
            }
        });
    }

    _updateIndicators() {
        Object.entries(this._indicators || {}).forEach(([key, indicator]) => {
            if (!indicator) return;
            this._updateIndicatorVisibility(key, indicator);
        });
    }

    _updateIndicatorVisibility(key, indicator) {
        if (!indicator) return;
        const target = indicator._indicator || indicator;
        const mode = this._settings.get_string(key);
    
        if (mode === 'hidden') {
            target.hide();
        } else if (mode === 'shown') {
            target.show();
        }
    }
}